@@ -5,6 +5,8 @@ use serde::{Deserialize, Serialize};
 
 use crate::handle::{JointNamesAndPositions, RobotOrigin, RobotStateHandle};
 
+type Handle = web::Data<Arc<RobotStateHandle>>;
+
 #[derive(Deserialize, Serialize, Debug, Clone)]
 struct ResultResponse {
     is_ok: bool,
@@ -39,8 +41,10 @@ impl WebServer {
                 .service(options_set_robot_origin)
                 .service(get_joint_positions)
                 .service(get_robot_origin)
+                .service(get_urdf_text)
                 .service(options_get_joint_positions)
                 .service(options_get_robot_origin)
+                .service(options_get_urdf_text)
         })
         .bind(("0.0.0.0", self.port))?
         .run()
@@ -51,9 +55,8 @@ impl WebServer {
 #[post("set_joint_positions")]
 async fn set_joint_positions(
     json: web::Json<JointNamesAndPositions>,
-    handle: web::Data<Arc<RobotStateHandle>>,
+    handle: Handle,
 ) -> HttpResponse {
-    let mut jp_request = handle.target_joint_positions.lock().unwrap();
     let jp: JointNamesAndPositions = json.into_inner();
     if jp.names.len() != jp.positions.len() {
         HttpResponse::Ok().json(&ResultResponse {
@@ -65,7 +68,7 @@ async fn set_joint_positions(
             ),
         })
     } else {
-        *jp_request = Some(jp);
+        handle.set_target_joint_positions(jp);
         HttpResponse::Ok().json(&ResultResponse {
             is_ok: true,
             reason: "".to_string(),
@@ -74,12 +77,8 @@ async fn set_joint_positions(
 }
 
 #[post("set_robot_origin")]
-async fn set_robot_origin(
-    json: web::Json<RobotOrigin>,
-    handle: web::Data<Arc<RobotStateHandle>>,
-) -> HttpResponse {
-    let mut pose_request = handle.target_robot_origin.lock().unwrap();
-    *pose_request = Some(json.into_inner());
+async fn set_robot_origin(json: web::Json<RobotOrigin>, handle: Handle) -> HttpResponse {
+    handle.set_target_robot_origin(json.into_inner());
     HttpResponse::Ok().json(&ResultResponse {
         is_ok: true,
         reason: "".to_string(),
@@ -109,17 +108,25 @@ async fn options_set_robot_origin() -> HttpResponse {
 }
 
 #[get("get_joint_positions")]
-async fn get_joint_positions(handle: web::Data<Arc<RobotStateHandle>>) -> HttpResponse {
-    let json = handle.current_joint_positions.lock().unwrap();
+async fn get_joint_positions(handle: Handle) -> HttpResponse {
+    let json = handle.current_joint_positions();
     HttpResponse::Ok().json(&*json)
 }
 
 #[get("get_robot_origin")]
-async fn get_robot_origin(handle: web::Data<Arc<RobotStateHandle>>) -> HttpResponse {
-    let origin = handle.current_robot_origin.lock().unwrap();
+async fn get_robot_origin(handle: Handle) -> HttpResponse {
+    let origin = handle.current_robot_origin();
     HttpResponse::Ok().json(&*origin)
 }
 
+#[get("get_urdf_text")]
+async fn get_urdf_text(handle: Handle) -> HttpResponse {
+    match handle.urdf_text() {
+        Some(text) => HttpResponse::from(&*text),
+        None => HttpResponse::InternalServerError().finish(),
+    }
+}
+
 #[options("get_joint_positions")]
 async fn options_get_joint_positions() -> HttpResponse {
     HttpResponse::NoContent()
@@ -141,3 +148,14 @@ async fn options_get_robot_origin() -> HttpResponse {
         .header("Access-Control-Max-Age", "86400")
         .finish()
 }
+
+#[options("get_urdf_text")]
+async fn options_get_urdf_text() -> HttpResponse {
+    HttpResponse::NoContent()
+        .header("Allow", "OPTIONS, GET")
+        .header("Access-Control-Allow-Methods", "GET")
+        .header("Access-Control-Allow-Origin", "*")
+        .header("Access-Control-Allow-Headers", "authorization")
+        .header("Access-Control-Max-Age", "86400")
+        .finish()
+}
